/********************************************************************************
*                                                                               *
*                      F X   M O V I E   M A N A G E R                          *
*                                                                               *
*********************************************************************************
* Copyright(C) 2008,2009 by Luigi Ziviani <ziviani_luigi@yahoo.it>.             *
* All Rights Reserved.                                                          *
*********************************************************************************
* Id: mimetypedialog.cpp,   v 2.0                      2008/01/01               *
*********************************************************************************
* This program is free software; you can redistribute it and/or modify          *
* it under the terms of the GNU General Public License as published by          *
* the Free Software Foundation; either version 3 of the License, or             *
* (at your option) any later version.                                           *
*                                                                               *
* This program is distributed in the hope that it will be useful,               *
* but WITHOUT ANY WARRANTY; without even the implied warranty of                *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                 *
* GNU General Public License for more details.                                  *
* You should have received a copy of the GNU General Public License             *
* along with this program; if not, write to the Free Software                   *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA     * 
********************************************************************************/

#include <xincs.h>
#include <fx.h>
#include "MimeTypeDialog.h"
#include "fxmoviemanager.h"
#include "icons.h"
#include "config.h"


FXDEFMAP(MimeTypeDialog) MimeTypeDialogMap[]={

  FXMAPFUNC(SEL_COMMAND,MimeTypeDialog::ID_SELECT_ICONPATH,		MimeTypeDialog::onCmdSelectIconPath),
  FXMAPFUNC(SEL_COMMAND,MimeTypeDialog::ID_SELECT_COMMAND,		MimeTypeDialog::onCmdSelectCommand),
  FXMAPFUNC(SEL_COMMAND,MimeTypeDialog::ID_SELECT_MIMETYPE,		MimeTypeDialog::onCmdSelectMimeType),
  FXMAPFUNC(SEL_CLOSE,	MimeTypeDialog::ID_FXMMAPPLICATION,		MimeTypeDialog::onCmdCloseDialog),
  FXMAPFUNC(SEL_COMMAND,MimeTypeDialog::ID_CREATE_MIMEASSOCIATION,	MimeTypeDialog::onCmdCreateFileAssociation),
  FXMAPFUNC(SEL_COMMAND,MimeTypeDialog::ID_DELETE_MIMEASSOCIATION,	MimeTypeDialog::onCmdDeleteFileAssociation),
  FXMAPFUNC(SEL_COMMAND,MimeTypeDialog::ID_RENAME_MIMEASSOCIATION,	MimeTypeDialog::onCmdRenameFileAssociation),
  FXMAPFUNC(SEL_CHANGED,MimeTypeDialog::ID_SELECT_MIMEASSOCIATION,	MimeTypeDialog::onCmdFileAssociation),
  FXMAPFUNC(SEL_COMMAND,MimeTypeDialog::ID_SELECT_TERMINAL,		MimeTypeDialog::onCmdSelectTerminal),
  FXMAPFUNC(SEL_COMMAND,MimeTypeDialog::ID_SELECT_FONT,			MimeTypeDialog::onCmdSelectFont),
  FXMAPFUNCS(SEL_COMMAND,MimeTypeDialog::ID_SELECT_THEME_DARK,		MimeTypeDialog::ID_SELECT_THEME_MOON,		MimeTypeDialog::onCmdSelectThemeColors),
  FXMAPFUNCS(SEL_COMMAND,MimeTypeDialog::ID_SELECT_LANGUAGE_EN,		MimeTypeDialog::ID_SELECT_LANGUAGE_RU,		MimeTypeDialog::onCmdSelectLanguage),
  FXMAPFUNCS(SEL_COMMAND,MimeTypeDialog::ID_SELECT_ICONBIG_NAME,	MimeTypeDialog::ID_SELECT_ICONSMALL_NAME,	MimeTypeDialog::onCmdSelectIcon),
  };


// Object implementation
FXIMPLEMENT(MimeTypeDialog,FXDialogBox,MimeTypeDialogMap,ARRAYNUMBER(MimeTypeDialogMap))


// Construct window
MimeTypeDialog::MimeTypeDialog(FXWindow *own):FXDialogBox(own,"FXMovieManager Setup",DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE,0,0,0,0){

  // Default font
  bodyfont=NULL;

  // get iconpath, theme and language Settings
  fxmmwheellines = getApp()->reg().readIntEntry("SETTINGS","fxmmwheellines",4);
  iconpath = getApp()->reg().readStringEntry("SETTINGS","mimeiconpath");
  theme = getApp()->reg().readStringEntry("SETTINGS","theme");
  language = getApp()->reg().readStringEntry("SETTINGS","language");
  terminal = getApp()->reg().readStringEntry("SETTINGS","terminal");
  fontspec = getApp()->reg().readStringEntry("SETTINGS","bodyfont","");
  if(!fontspec.empty()){
    bodyfont=new FXFont(getApp(),fontspec);
    }

#ifdef HAVE_LIBFFMPEGTHUMBNAILER
  // get thumbnailer options
  thumbnailSize = getApp()->reg().readIntEntry("THUMBNAILER","thumbnailSize",128);
  imageQuality = getApp()->reg().readIntEntry("THUMBNAILER","imageQuality",8);
  filmStripOverlay = getApp()->reg().readBoolEntry("THUMBNAILER","filmStripOverlay",FALSE);
  workaroundIssues = getApp()->reg().readBoolEntry("THUMBNAILER","workaroundIssues",FALSE);
  maintainAspectRatio = getApp()->reg().readBoolEntry("THUMBNAILER","maintainAspectRatio",TRUE);
  smartFrameSelection = getApp()->reg().readBoolEntry("THUMBNAILER","smartFrameSelection",FALSE);
#endif
  autoThumbnailShow = getApp()->reg().readBoolEntry("THUMBNAILER","autoThumbnailShow",TRUE);
  autoThumbnailCreate = getApp()->reg().readBoolEntry("THUMBNAILER","autoThumbnailCreate",FALSE);

  // Set the Main Application Target
  setTarget(this);

  // Connect Datatargets
  target_association_type.connect(fileassociation.type);
  target_association_command.connect(fileassociation.command);
  target_association_mimetype.connect(fileassociation.mimetype);
  target_iconpath.connect(iconpath);
  target_terminal.connect(terminal);
  target_wheel.connect(fxmmwheellines);

#ifdef HAVE_LIBFFMPEGTHUMBNAILER
  target_thumbnailSize.connect(thumbnailSize);
  target_imageQuality.connect(imageQuality);
  target_filmStripOverlay.connect(filmStripOverlay);
  target_workaroundIssues.connect(workaroundIssues);
  target_maintainAspectRatio.connect(maintainAspectRatio);
  target_smartFrameSelection.connect(smartFrameSelection);
#endif
  target_autoThumbnailShow.connect(autoThumbnailShow);
  target_autoThumbnailCreate.connect(autoThumbnailCreate);

  // Set the message identifier for this data target
  setSelector(ID_FXMMAPPLICATION);

  // Retrive theme settings
  theme=getApp()->reg().readStringEntry("SETTINGS","theme");
  theme_fxmmbase=getApp()->reg().readColorEntry("SETTINGS","fxmmbasecolor");
  theme_fxmmborder=getApp()->reg().readColorEntry("SETTINGS","fxmmbordercolor");
  theme_fxmmfore=getApp()->reg().readColorEntry("SETTINGS","fxmmforecolor");
  theme_fxmmback=getApp()->reg().readColorEntry("SETTINGS","fxmmbackcolor");
  theme_fxmmmenufore=getApp()->reg().readColorEntry("SETTINGS","fxmmmenuforecolor");
  theme_fxmmmenuback=getApp()->reg().readColorEntry("SETTINGS","fxmmmenubackcolor");
  theme_fxmmselfore=getApp()->reg().readColorEntry("SETTINGS","fxmmselforecolor");
  theme_fxmmselback=getApp()->reg().readColorEntry("SETTINGS","fxmmselbackcolor");
  theme_fxmmtipfore=getApp()->reg().readColorEntry("SETTINGS","fxmmtipforecolor");
  theme_fxmmtipback=getApp()->reg().readColorEntry("SETTINGS","fxmmtipbackcolor");
  theme_fxmmhilite=getApp()->reg().readColorEntry("SETTINGS","fxmmhilitecolor");
  theme_fxmmshadow=getApp()->reg().readColorEntry("SETTINGS","fxmmshadowcolor");

  // Build icons
  logoicon=new FXPNGIcon(getApp(),fxmmlogo);
  abouticon=new FXGIFIcon(getApp(),b_list);
  conficon=new FXGIFIcon(getApp(),b_conf);
  diricon=new FXGIFIcon(getApp(),m_dir);
  fonticon=new FXGIFIcon(getApp(),m_renamedir);

  darkthemeicon=new FXJPGIcon(getApp(),darktheme);
  shinethemeicon=new FXJPGIcon(getApp(),shinetheme);
  classicthemeicon=new FXJPGIcon(getApp(),classictheme);
  gentlethemeicon=new FXJPGIcon(getApp(),gentletheme);
  fogthemeicon=new FXJPGIcon(getApp(),fogtheme);
  moonthemeicon=new FXJPGIcon(getApp(),moontheme);

  en_flagicon=new FXGIFIcon(getApp(),en_flag);
  it_flagicon=new FXGIFIcon(getApp(),it_flag);
  de_flagicon=new FXGIFIcon(getApp(),de_flag);
  fr_flagicon=new FXGIFIcon(getApp(),fr_flag);
  es_flagicon=new FXGIFIcon(getApp(),es_flag);
  ru_flagicon=new FXGIFIcon(getApp(),ru_flag);

  cancelicon=new FXGIFIcon(getApp(),m_cancel);
  accepticon=new FXGIFIcon(getApp(),m_accept);

  // get the size of normal font and create a new font with size x 1.5
  getApp()->getNormalFont()->create();
  FXFontDesc fontdescription=getApp()->getNormalFont()->getFontDesc();
  fontdescription.size = (FXuint) (((double)fontdescription.size) * 1.5);
  titlefont = new FXFont(getApp(),fontdescription);
  titlefont->create();

  tooltip=new FXToolTip(getApp());

  // Declare reused frames
  FXHorizontalFrame *hframe=NULL;
  FXVerticalFrame   *sframe=NULL;
  FXVerticalFrame   *vframe=NULL;
  FXLabel           *label=NULL;
  FXPacker 	    *packer;

  // main frame
  FXVerticalFrame   *mainframe=new FXVerticalFrame(this,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,0,0,0,0,0,0);

  // Get icon margin
  const FXint spacing=(logoicon ? 10 : 20);

  // Create the header
  label=new FXLabel(mainframe," FXMovieManager Configuration",logoicon,LAYOUT_FILL_X|JUSTIFY_LEFT|TEXT_AFTER_ICON,0,0,0,0,spacing,spacing,spacing,spacing);
  label->setBackColor(FXRGB(0,0,0));
  label->setTextColor(FXRGB(255,255,0));
  label->setFont(titlefont);

  new FXSeparator(mainframe,SEPARATOR_GROOVE|LAYOUT_FILL_X);

  
  FXHorizontalFrame *hmainframe=new FXHorizontalFrame(mainframe,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,0,0,0,0,0,0);
  FXVerticalFrame *buttonframe=new FXVerticalFrame(hmainframe,LAYOUT_FILL_Y|LAYOUT_LEFT|PACK_UNIFORM_WIDTH|PACK_UNIFORM_HEIGHT,0,0,0,0, 5,5,5,5);
  new FXSeparator(hmainframe,SEPARATOR_GROOVE|LAYOUT_FILL_Y);


  FXSwitcher *switcher = new FXSwitcher(hmainframe,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,0,0,0,0);

  vframe = new FXVerticalFrame(buttonframe,FRAME_SUNKEN,0,0,0,0,0,0,0,0);
  new FXButton(vframe,"About",abouticon,switcher,FXSwitcher::ID_OPEN_FIRST,FRAME_RAISED|ICON_ABOVE_TEXT|LAYOUT_FILL);
  vframe = new FXVerticalFrame(buttonframe,FRAME_SUNKEN,0,0,0,0,0,0,0,0);
  new FXButton(vframe,"HowTo",abouticon,switcher,FXSwitcher::ID_OPEN_SECOND,FRAME_RAISED|ICON_ABOVE_TEXT|LAYOUT_FILL);
  vframe = new FXVerticalFrame(buttonframe,FRAME_SUNKEN,0,0,0,0,0,0,0,0);
  new FXButton(vframe,"Keyboard",abouticon,switcher,FXSwitcher::ID_OPEN_THIRD,FRAME_RAISED|ICON_ABOVE_TEXT|LAYOUT_FILL);
  vframe = new FXVerticalFrame(buttonframe,FRAME_SUNKEN,0,0,0,0,0,0,0,0);
  new FXButton(vframe,"MimeTypes",conficon,switcher,FXSwitcher::ID_OPEN_FOURTH,FRAME_RAISED|ICON_ABOVE_TEXT|LAYOUT_FILL);
  vframe = new FXVerticalFrame(buttonframe,FRAME_SUNKEN,0,0,0,0,0,0,0,0);
  new FXButton(vframe,"Configure",conficon,switcher,FXSwitcher::ID_OPEN_FIFTH,FRAME_RAISED|ICON_ABOVE_TEXT|LAYOUT_FILL);
  vframe = new FXVerticalFrame(buttonframe,FRAME_SUNKEN,0,0,0,0,0,0,0,0);
  new FXButton(vframe,"Thumbnail",conficon,switcher,FXSwitcher::ID_OPEN_SIXTH,FRAME_RAISED|ICON_ABOVE_TEXT|LAYOUT_FILL);


  // About this program
  vframe = new FXVerticalFrame(switcher,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,0,0,0,0,0,0);

  hframe = new FXHorizontalFrame(vframe,LAYOUT_FILL_X,0,0,0,0,0,0,0,0,0,0);
  new FXLabel(hframe," About FXMovieManager",NULL,JUSTIFY_CENTER_Y);

  new FXSeparator(vframe,SEPARATOR_GROOVE|LAYOUT_FILL_X);

  label = new FXLabel(vframe,
	"FXMovieManager is a FileManager with movie thumbnails,\n"
	"it manages a PlayList and act as simple front-end to MPlayer.\n"
	"Copyright (C) 2008,2009 Luigi Ziviani (ziviani_luigi@yahoo.it).\n\n"
	"This program is free software; you can redistribute it\n"
	"and/or modify it under the terms of the GNU General Public\n"
	"License as published by the Free Software Foundation;\n"
	"either version 3 of the License, or(at your option)\n"
	"any later version.\n\n"
	"This program is distributed in the hope that it will be useful,\n"
	"but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
	"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n"
	"See the GNU General Public License for more details.\n\n"
	"You should have received a copy of the GNU General Public License\n"
	"along with this program; if not, write to the \n"
	"Free Software Foundation, Inc.,\n"
	"59 Temple Place, Suite 330, Boston, MA  02111-1307  USA"
	);



  // Help instructions
  vframe = new FXVerticalFrame(switcher,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,0,0,0,0,0,0);

  hframe = new FXHorizontalFrame(vframe,LAYOUT_FILL_X,0,0,0,0,0,0,0,0,0,0);
  new FXLabel(hframe," How to use FXMovieManager",NULL,JUSTIFY_CENTER_Y);

  new FXSeparator(vframe,SEPARATOR_GROOVE|LAYOUT_FILL_X);

  label = new FXLabel(vframe,
	" To manage thumbnails of movie files a list of buttons on the\n"
	" right of the main application area can be used.\n\n"
	" 1)  To Create and Show thumbnails of movie files:\n"
	" select the movies and press the button [Create thumbnails].\n"
	" 2)  To ReCreate the thumbnails:\n"
	" select the movies and press the button [Recreate thumbnails].\n"
	" 3)  To load a movie into the playlist:\n"
	" double click on a movie in the filelist or open the file dialog\n"
	" and select one or more movies keeping [Ctrl] or [Shift] pressed.\n"
	" 4)  To delete hidden directories that store the thumbnails:\n"
	" press the button [Delete Thumbnails]\n"
	" 5)  To associate file extensions to icons, mime-types and commands:\n"
	" Select the button [MimeTypes] in the Current dialog and\n"
	" a window will be displayed where to add new file associations\n"
	" 6)  To change Theme/Language/Font/Terminal:\n"
	" Select the button  [Configure] in the Current dialog and\n"
	" a window will be displayed where to set your preferences\n"
	" 7)  To manages files: right mouse click on the file/files selected\n"
	,NULL,JUSTIFY_LEFT);



  // Short Cut instructions
  vframe = new FXVerticalFrame(switcher,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,0,0,0,0,0,0);

  hframe = new FXHorizontalFrame(vframe,LAYOUT_FILL_X,0,0,0,0,0,0,0,0,0,0);
  new FXLabel(hframe," Some keyboard controls are enabled",NULL,JUSTIFY_CENTER_Y);

  new FXSeparator(vframe,SEPARATOR_GROOVE|LAYOUT_FILL_X);

  label = new FXLabel(vframe,
	" F1 = show help and configuration dialog\n\n"
	" Other Function buttons: CTRL manages LEFT Panel\n"
	" F2 = reverse file selection\n"
	" F3 = create thunbnails of selected files\n"
	" F4 = force thumbnails recreation of selected files\n"
	" F5 = increase thumbnails size of selected files\n"
	" F6 = decrease thumbnails size of selected files\n"
	" F7 = show big icon list\n"
	" F8 = show small icon list\n\n"
	" Other Keys: CTRL manages LEFT Panel   ALT manages RIGHT Panel\n"
	" C or c = copy\n"
	" S or s = symlink\n"
	" R or r = rename\n"
	" Z or z = play\n"
	" A or a = pause\n"
	" Q or q = quit\n" 
	,NULL,JUSTIFY_LEFT);

  new FXSeparator(vframe,SEPARATOR_GROOVE|LAYOUT_FILL_X);

  new FXLabel(vframe,"Mouse Wheel Speed",NULL,LAYOUT_LEFT|LAYOUT_CENTER_Y);
  mousespinner=new FXSpinner(vframe,4,&target_wheel,FXDataTarget::ID_VALUE,FRAME_SUNKEN|FRAME_THICK);
  mousespinner->setRange(1,20);



  // Content-Types configuration
  vframe = new FXVerticalFrame(switcher,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,0,0,0,0,0,0);

  hframe = new FXHorizontalFrame(vframe,LAYOUT_FILL_X,0,0,0,0,5,5,5,5);
  new FXLabel(hframe,"Search Path for Icons:",NULL,LAYOUT_CENTER_Y);
  new FXTextField(hframe,2,&target_iconpath,FXDataTarget::ID_VALUE,LAYOUT_SIDE_LEFT|LAYOUT_FILL_X|LAYOUT_CENTER_Y|FRAME_SUNKEN|FRAME_THICK);
  new FXButton(hframe,"..",diricon,this,ID_SELECT_ICONPATH,LAYOUT_SIDE_RIGHT|LAYOUT_CENTER_Y|ICON_AFTER_TEXT|FRAME_RAISED|FRAME_THICK);

  new FXSeparator(vframe,SEPARATOR_GROOVE|LAYOUT_FILL_X);

  hframe = new FXHorizontalFrame(vframe,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,0,0,0,0,0,0);
  vframe = new FXVerticalFrame(hframe,LAYOUT_FILL_Y,0,0,0,0,5,5,5,5);

  new FXSeparator(hframe,SEPARATOR_GROOVE|LAYOUT_FILL_Y);

  new FXLabel(vframe,"Extensions of File: ");
  sframe = new FXVerticalFrame(vframe,FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_Y|LAYOUT_FILL_X,0,0,0,0,0,0,0,0);
  associationlist = new FXList(sframe,this,ID_SELECT_MIMEASSOCIATION,LAYOUT_FILL_Y|LAYOUT_FILL_X|LIST_BROWSESELECT);
  associationlist->setSortFunc(FXList::ascending);
  FXHorizontalFrame * listbuttonframe = new FXHorizontalFrame(vframe,PACK_UNIFORM_WIDTH|LAYOUT_FILL_X,0,0,0,0,0,0,0,0);
  new FXButton(listbuttonframe,"&Insert\tInsert a New Extension",NULL,this,ID_CREATE_MIMEASSOCIATION);
  new FXButton(listbuttonframe,"&Delete\tDelete an Extension",NULL,this,ID_DELETE_MIMEASSOCIATION);
  new FXButton(listbuttonframe,"&Change\tChange the name of an Extension",NULL,this,ID_RENAME_MIMEASSOCIATION);

  vframe = new FXVerticalFrame(hframe,LAYOUT_FILL_Y|LAYOUT_FILL_X,0,0,0,0,0,0,0,0,0,0);


  packer = new FXPacker(vframe,LAYOUT_FILL_X,0,0,0,0,5,5,5,5);
  new FXLabel(packer,"Type Name:");
  new FXTextField(packer,2,&target_association_type,FXDataTarget::ID_VALUE,LAYOUT_SIDE_LEFT|LAYOUT_FILL_X|LAYOUT_CENTER_Y|FRAME_SUNKEN|FRAME_THICK);

  new FXSeparator(vframe,SEPARATOR_GROOVE|LAYOUT_FILL_X);


  packer = new FXPacker(vframe,LAYOUT_FILL_X,0,0,0,0,5,5,5,5);
  new FXLabel(packer,"Content-Type [object/format]:");
  mimetypelist = new FXComboBox(packer,1,this,ID_SELECT_MIMETYPE,LAYOUT_FILL_X|FRAME_SUNKEN|FRAME_THICK|COMBOBOX_NO_REPLACE);
  mimetypelist->setSortFunc(FXList::ascending);
  mimetypelist->setNumVisible(9);
  mimetypelist->appendItem(" ");

  new FXSeparator(vframe,SEPARATOR_GROOVE|LAYOUT_FILL_X);


  packer = new FXPacker(vframe,LAYOUT_FILL_X,0,0,0,0,5,5,5,5);
  new FXLabel(packer,"Open With Command:");
  new FXButton(packer,"..",diricon,this,ID_SELECT_COMMAND,LAYOUT_SIDE_RIGHT|LAYOUT_CENTER_Y|ICON_AFTER_TEXT|FRAME_RAISED|FRAME_THICK);
  new FXTextField(packer,2,&target_association_command,FXDataTarget::ID_VALUE,LAYOUT_SIDE_LEFT|LAYOUT_FILL_X|LAYOUT_CENTER_Y|FRAME_SUNKEN|FRAME_THICK);

  new FXSeparator(vframe,SEPARATOR_GROOVE|LAYOUT_FILL_X);


  FXMatrix *iconsmatrix=new FXMatrix(vframe,4,MATRIX_BY_COLUMNS|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0, 5,5,5,5);
  new FXLabel(iconsmatrix,"Normal",NULL,LAYOUT_CENTER_X|LAYOUT_FILL_COLUMN);
  button_bigname = new FXButton(iconsmatrix,"\tChange MIME-Type Icon",NULL,this,ID_SELECT_ICONBIG_NAME,FRAME_RAISED|LAYOUT_CENTER_X|LAYOUT_CENTER_Y|JUSTIFY_CENTER_X|JUSTIFY_CENTER_Y|LAYOUT_FILL_COLUMN|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,64,64, 1,1,1,1);
  new FXLabel(iconsmatrix,"Small",NULL,LAYOUT_CENTER_X|LAYOUT_FILL_COLUMN);
  button_smallname = new FXButton(iconsmatrix,"\tChange MIME-Type Icon",NULL,this,ID_SELECT_ICONSMALL_NAME,FRAME_RAISED|LAYOUT_CENTER_X|LAYOUT_CENTER_Y|JUSTIFY_CENTER_X|JUSTIFY_CENTER_Y|LAYOUT_FILL_COLUMN|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,64,64, 1,1,1,1);


  // import file associations **********************************************//
  FXString datastring;
  FXString mimetype;

  FXStringDict *filetypes; // The dictionary class maintains a fast-access 
                           // hash table of entities indexed by a character string.

  filetypes = getApp()->reg().find("FILETYPES"); // FILETYPES is the registry section where are contents-type data

  if(filetypes){
    for(FXint i=filetypes->first(); i<filetypes->size(); i=filetypes->next(i)){
      associationlist->appendItem(filetypes->key(i),NULL,NULL,TRUE); 	// Key string of struct FXDictEntry
      datastring = filetypes->data(i); 					// Data of struct FXDictEntry
      mimetype = datastring.section(";",4);
      if (!mimetype.empty() && (mimetypelist->findItem(mimetype)==-1)){
        mimetypelist->appendItem(mimetype);
        }
      }
    }

  associationlist->sortItems();
  mimetypelist->sortItems();
  //***********************************************************************//



  // Preferences
  FXMatrix *framematrix=new FXMatrix(switcher,2,MATRIX_BY_COLUMNS|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0, 5,5,5,5);

  new FXLabel(framematrix," Theme selection",NULL,LAYOUT_SIDE_TOP|LAYOUT_CENTER_X);
  new FXLabel(framematrix," Language selection",NULL,LAYOUT_SIDE_TOP|LAYOUT_CENTER_X);


  FXMatrix *themesmatrix=new FXMatrix(framematrix,2,MATRIX_BY_COLUMNS|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0, 5,5,5,5);
  darkthemebutton=new FXButton(themesmatrix,"\tDark Theme",NULL,this,ID_SELECT_THEME_DARK,FRAME_RAISED|LAYOUT_CENTER_X|LAYOUT_CENTER_Y|JUSTIFY_CENTER_X|JUSTIFY_CENTER_Y|LAYOUT_FILL_COLUMN|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,100,80, 1,1,1,1);
  darkthemebutton->setIcon(darkthemeicon);
  gentlethemebutton=new FXButton(themesmatrix,"\tGentle Theme",NULL,this,ID_SELECT_THEME_GENTLE,FRAME_RAISED|LAYOUT_CENTER_X|LAYOUT_CENTER_Y|JUSTIFY_CENTER_X|JUSTIFY_CENTER_Y|LAYOUT_FILL_COLUMN|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,100,80, 1,1,1,1);
  gentlethemebutton->setIcon(gentlethemeicon);
  fogthemebutton=new FXButton(themesmatrix,"\tFog Theme",NULL,this,ID_SELECT_THEME_FOG,FRAME_RAISED|LAYOUT_CENTER_X|LAYOUT_CENTER_Y|JUSTIFY_CENTER_X|JUSTIFY_CENTER_Y|LAYOUT_FILL_COLUMN|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,100,80, 1,1,1,1);
  fogthemebutton->setIcon(fogthemeicon);
  shinethemebutton=new FXButton(themesmatrix,"\tShine Theme",NULL,this,ID_SELECT_THEME_SHINE,FRAME_RAISED|LAYOUT_CENTER_X|LAYOUT_CENTER_Y|JUSTIFY_CENTER_X|JUSTIFY_CENTER_Y|LAYOUT_FILL_COLUMN|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,100,80, 1,1,1,1);
  shinethemebutton->setIcon(shinethemeicon);
  moonthemebutton=new FXButton(themesmatrix,"\tMoon Theme",NULL,this,ID_SELECT_THEME_MOON,FRAME_RAISED|LAYOUT_CENTER_X|LAYOUT_CENTER_Y|JUSTIFY_CENTER_X|JUSTIFY_CENTER_Y|LAYOUT_FILL_COLUMN|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,100,80, 1,1,1,1);
  moonthemebutton->setIcon(moonthemeicon);
  classicthemebutton=new FXButton(themesmatrix,"\tClassic Theme",NULL,this,ID_SELECT_THEME_CLASSIC,FRAME_RAISED|LAYOUT_CENTER_X|LAYOUT_CENTER_Y|JUSTIFY_CENTER_X|JUSTIFY_CENTER_Y|LAYOUT_FILL_COLUMN|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,100,80, 1,1,1,1);
  classicthemebutton->setIcon(classicthemeicon);

  FXMatrix *languagesmatrix=new FXMatrix(framematrix,2,MATRIX_BY_COLUMNS|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0, 5,5,5,5);
  en_button=new FXButton(languagesmatrix,"\tEN language",NULL,this,ID_SELECT_LANGUAGE_EN,FRAME_RAISED|LAYOUT_CENTER_X|LAYOUT_CENTER_Y|JUSTIFY_CENTER_X|JUSTIFY_CENTER_Y|LAYOUT_FILL_COLUMN|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,100,80, 1,1,1,1);
  en_button->setIcon(en_flagicon);
  it_button=new FXButton(languagesmatrix,"\tIT language",NULL,this,ID_SELECT_LANGUAGE_IT,FRAME_RAISED|LAYOUT_CENTER_X|LAYOUT_CENTER_Y|JUSTIFY_CENTER_X|JUSTIFY_CENTER_Y|LAYOUT_FILL_COLUMN|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,100,80, 1,1,1,1);
  it_button->setIcon(it_flagicon);
  de_button=new FXButton(languagesmatrix,"\tDE language",NULL,this,ID_SELECT_LANGUAGE_DE,FRAME_RAISED|LAYOUT_CENTER_X|LAYOUT_CENTER_Y|JUSTIFY_CENTER_X|JUSTIFY_CENTER_Y|LAYOUT_FILL_COLUMN|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,100,80, 1,1,1,1);
  de_button->setIcon(de_flagicon);
  fr_button=new FXButton(languagesmatrix,"\tFR language",NULL,this,ID_SELECT_LANGUAGE_FR,FRAME_RAISED|LAYOUT_CENTER_X|LAYOUT_CENTER_Y|JUSTIFY_CENTER_X|JUSTIFY_CENTER_Y|LAYOUT_FILL_COLUMN|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,100,80, 1,1,1,1);
  fr_button->setIcon(fr_flagicon);
  es_button=new FXButton(languagesmatrix,"\tES language",NULL,this,ID_SELECT_LANGUAGE_ES,FRAME_RAISED|LAYOUT_CENTER_X|LAYOUT_CENTER_Y|JUSTIFY_CENTER_X|JUSTIFY_CENTER_Y|LAYOUT_FILL_COLUMN|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,100,80, 1,1,1,1);
  es_button->setIcon(es_flagicon);
  ru_button=new FXButton(languagesmatrix,"\tRU language",NULL,this,ID_SELECT_LANGUAGE_RU,FRAME_RAISED|LAYOUT_CENTER_X|LAYOUT_CENTER_Y|JUSTIFY_CENTER_X|JUSTIFY_CENTER_Y|LAYOUT_FILL_COLUMN|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,100,80, 1,1,1,1);
  ru_button->setIcon(ru_flagicon);


  new FXLabel(framematrix,"Font selection",NULL,LAYOUT_SIDE_TOP|LAYOUT_CENTER_X);
  new FXLabel(framematrix,"Terminal selection",NULL,LAYOUT_SIDE_TOP|LAYOUT_CENTER_X);


  fontbutton = new FXButton(framematrix,"Click to select a FONT ",fonticon,this,ID_SELECT_FONT,LAYOUT_SIDE_RIGHT|LAYOUT_FILL_X|LAYOUT_CENTER_Y|ICON_AFTER_TEXT|FRAME_RAISED|FRAME_THICK);
  if(!fontspec.empty()){
    fontbutton->setFont(bodyfont);
    }

  hframe = new FXHorizontalFrame(framematrix,LAYOUT_FILL_X,0,0,0,0,5,5,5,5);
  new FXTextField(hframe,2,&target_terminal,FXDataTarget::ID_VALUE,LAYOUT_SIDE_LEFT|LAYOUT_FILL_X|LAYOUT_CENTER_Y|FRAME_SUNKEN|FRAME_THICK);
  new FXButton(hframe,"..",diricon,this,ID_SELECT_TERMINAL,LAYOUT_SIDE_RIGHT|LAYOUT_CENTER_Y|ICON_AFTER_TEXT|FRAME_RAISED|FRAME_THICK);
  //***********************************************************************//



  // Thumbnailer
  vframe = new FXVerticalFrame(switcher,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,0,0,0,0,0,0);

#ifdef HAVE_LIBFFMPEGTHUMBNAILER
  new FXLabel(vframe," ffmpegthumbnailer options",NULL,JUSTIFY_CENTER_Y);
  new FXSeparator(vframe,SEPARATOR_GROOVE|LAYOUT_FILL_X);

  FXMatrix *thumbmatrix1=new FXMatrix(vframe,2,MATRIX_BY_COLUMNS|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0, 5,5,5,5);

  new FXLabel(thumbmatrix1," thumbnail Size",NULL,LAYOUT_SIDE_TOP|LAYOUT_CENTER_X);
  new FXTextField(thumbmatrix1,5,&target_thumbnailSize,FXDataTarget::ID_VALUE,LAYOUT_SIDE_LEFT|LAYOUT_FILL_X|LAYOUT_CENTER_Y|FRAME_SUNKEN|FRAME_THICK);

  new FXLabel(thumbmatrix1," image Quality",NULL,LAYOUT_SIDE_TOP|LAYOUT_CENTER_X);
  FXRealSpinner *qualityspinner = new FXRealSpinner(thumbmatrix1,2,&target_imageQuality,FXDataTarget::ID_VALUE,REALSPIN_CYCLIC|FRAME_SUNKEN|FRAME_THICK|LAYOUT_SIDE_TOP|LAYOUT_FILL_X);
  qualityspinner->setRange(0,11);

  new FXLabel(thumbmatrix1," film Strip Overlay",NULL,LAYOUT_SIDE_TOP|LAYOUT_CENTER_X);
  new FXToggleButton(thumbmatrix1,"NO","YES",cancelicon,accepticon,&target_filmStripOverlay,FXDataTarget::ID_VALUE,BUTTON_TOOLBAR|ICON_BEFORE_TEXT|FRAME_RAISED|FRAME_THICK);

  new FXLabel(thumbmatrix1," workaround Issues",NULL,LAYOUT_SIDE_TOP|LAYOUT_CENTER_X);
  new FXToggleButton(thumbmatrix1,"NO","YES",cancelicon,accepticon,&target_workaroundIssues,FXDataTarget::ID_VALUE,BUTTON_TOOLBAR|ICON_BEFORE_TEXT|FRAME_RAISED|FRAME_THICK);

  new FXLabel(thumbmatrix1," maintain Aspect Ratio",NULL,LAYOUT_SIDE_TOP|LAYOUT_CENTER_X);
  new FXToggleButton(thumbmatrix1,"NO","YES",cancelicon,accepticon,&target_maintainAspectRatio,FXDataTarget::ID_VALUE,BUTTON_TOOLBAR|ICON_BEFORE_TEXT|FRAME_RAISED|FRAME_THICK);

  new FXLabel(thumbmatrix1," smart Frame Selection",NULL,LAYOUT_SIDE_TOP|LAYOUT_CENTER_X);
  new FXToggleButton(thumbmatrix1,"NO","YES",cancelicon,accepticon,&target_smartFrameSelection,FXDataTarget::ID_VALUE,BUTTON_TOOLBAR|ICON_BEFORE_TEXT|FRAME_RAISED|FRAME_THICK);

  new FXSeparator(vframe,SEPARATOR_GROOVE|LAYOUT_FILL_X);
#endif

  FXMatrix *thumbmatrix2=new FXMatrix(vframe,2,MATRIX_BY_COLUMNS|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0, 5,5,5,5);

  new FXLabel(thumbmatrix2," auto Thumbnail Show",NULL,LAYOUT_SIDE_TOP|LAYOUT_CENTER_X);
  new FXToggleButton(thumbmatrix2,"NO","YES",cancelicon,accepticon,&target_autoThumbnailShow,FXDataTarget::ID_VALUE,BUTTON_TOOLBAR|ICON_BEFORE_TEXT|FRAME_RAISED|FRAME_THICK);

  new FXLabel(thumbmatrix2," auto Thumbnail Create",NULL,LAYOUT_SIDE_TOP|LAYOUT_CENTER_X);
  new FXToggleButton(thumbmatrix2,"NO","YES",cancelicon,accepticon,&target_autoThumbnailCreate,FXDataTarget::ID_VALUE,BUTTON_TOOLBAR|ICON_BEFORE_TEXT|FRAME_RAISED|FRAME_THICK);


  new FXSeparator(vframe,SEPARATOR_GROOVE|LAYOUT_FILL_X);
  new FXLabel(vframe,"autoThumbnailCreate is only available if autoThumbnailShow = TRUE",NULL,TEXTFIELD_INTEGER|LAYOUT_SIDE_TOP|LAYOUT_CENTER_X);

  new FXSeparator(mainframe,SEPARATOR_GROOVE|LAYOUT_FILL_X);
  FXHorizontalFrame *copyright=new FXHorizontalFrame(mainframe,LAYOUT_BOTTOM|LAYOUT_FILL_X,0,0,0,0,5,5,5,5);
  label = new FXLabel(copyright,"FXMovieManager Settings - Copyright (C) Luigi Ziviani",NULL,LAYOUT_FILL_X|LAYOUT_CENTER_Y|JUSTIFY_LEFT,0,0,0,0,15);
  // set the same look of separators
  label->disable();

  new FXButton(copyright,"&Quit or Save",NULL,this,FXTopWindow::ID_CLOSE,BUTTON_INITIAL|BUTTON_DEFAULT|LAYOUT_RIGHT|FRAME_RAISED|FRAME_THICK,0,0,0,0,20,20);

  }


// Delete window
MimeTypeDialog::~MimeTypeDialog(){
  delete titlefont;
  delete bodyfont;
  delete logoicon;
  delete abouticon;
  delete conficon;
  delete diricon;
  delete darkthemeicon;
  delete shinethemeicon;
  delete classicthemeicon;
  delete gentlethemeicon;
  delete fogthemeicon;
  delete moonthemeicon;
  delete en_flagicon;
  delete it_flagicon;
  delete de_flagicon;
  delete fr_flagicon;
  delete es_flagicon;
  delete ru_flagicon;
  }


// Search for Icon and load it
static FXIcon *createIconApp(FXApp *app,const FXString& name,const FXString& iconpath){
  FXIconSource iconsource(app);
  FXIcon *icon=NULL;
  FXString fullpath;
  if(FXStat::exists(name)){
    fullpath=name;
    }
  else{
    fullpath=FXPath::search(iconpath,name);
    if(fullpath.empty()) return NULL;
    }
  icon=iconsource.loadIconFile(fullpath);
  if(icon){
    icon->blend(app->getBaseColor());
    icon->create();
    return icon;
    }
  return NULL;
  }


// Update Icon
static void updateIconApp(FXApp* app,FXButton* button,const FXString& path){
  FXIconSource iconsource(app);
  FXIcon *icon=NULL;
  if(button->getIcon()){
    icon=button->getIcon();
    button->setIcon(NULL);
    delete icon;
    icon=NULL;
    }
  icon=iconsource.loadIconFile(path);
  if(icon){
    icon->blend(app->getBaseColor());
    icon->create();
    button->setIcon(icon);
    }
  }


// Save file association entry
void MimeTypeDialog::saveFileAssociation(){
  if(fileassociation.extension.empty()) return;
  FXString added;
  added=fileassociation.command+";"+fileassociation.type+";"+fileassociation.bigname+";"+fileassociation.smallname+";"+fileassociation.mimetype;
  getApp()->reg().writeStringEntry("FILETYPES",fileassociation.extension.text(),added.text());
  }


// Save language
long MimeTypeDialog::onCmdSelectLanguage(FXObject*,FXSelector sel,void*){
    switch(FXSELID(sel)){
	case ID_SELECT_LANGUAGE_EN:
		language="en";
		break;
	case ID_SELECT_LANGUAGE_IT:
		language="it";
		break;
	case ID_SELECT_LANGUAGE_DE:
		language="de";
		break;
	case ID_SELECT_LANGUAGE_FR:
		language="fr";
		break;
	case ID_SELECT_LANGUAGE_ES:
		language="es";
		break;
	case ID_SELECT_LANGUAGE_RU:
		language="ru";
		break;
	}
    return 1;
}


// Save theme colors
long MimeTypeDialog::onCmdSelectThemeColors(FXObject*,FXSelector sel,void*){
    switch(FXSELID(sel)){
	case ID_SELECT_THEME_DARK:
  		theme="dark";
  		theme_fxmmfore   = FXRGB(200,200,200);
  		theme_fxmmbase   = FXRGB( 60, 60, 60);
  		theme_fxmmborder = FXRGB(  0,  0,  0);
  		theme_fxmmback   = FXRGB( 20, 20, 20);
  		theme_fxmmmenufore  = FXRGB(200,200,200);
  		theme_fxmmmenuback  = FXRGB( 10, 10, 10);
  		theme_fxmmselfore   = FXRGB(  0,  0,  0);
  		theme_fxmmselback   = FXRGB(100,100,100);
  		theme_fxmmtipfore  = FXRGB(200,200,200);
  		theme_fxmmtipback  = FXRGB( 20, 20, 20);
  		theme_fxmmhilite  = FXRGB(100,100,100);
  		theme_fxmmshadow  = FXRGB( 10, 10, 10);
	break;
	case ID_SELECT_THEME_SHINE:
  		theme="shine";
  		theme_fxmmfore   = FXRGB(100,100,100);
  		theme_fxmmbase   = FXRGB(230,230,230);
  		theme_fxmmborder = FXRGB( 80, 80, 80);
  		theme_fxmmback   = FXRGB(250,250,250);
  		theme_fxmmmenufore  = FXRGB( 50, 50, 50);
  		theme_fxmmmenuback  = FXRGB(180,180,180);
  		theme_fxmmselfore   = FXRGB( 50, 50, 50);
  		theme_fxmmselback   = FXRGB(200,200,200);
  		theme_fxmmtipfore  = FXRGB(100,100,100);
  		theme_fxmmtipback  = FXRGB(250,250,250);
  		theme_fxmmhilite  = FXRGB(250,250,250);
  		theme_fxmmshadow  = FXRGB(160,160,160);
	break;
	case ID_SELECT_THEME_CLASSIC:
 		theme="classic";
  		theme_fxmmfore   = FXRGB(  0,  0,  0);
  		theme_fxmmbase   = FXRGB(180,180,180);
  		theme_fxmmborder = FXRGB( 50, 50, 50);
  		theme_fxmmback   = FXRGB(210,210,210);
  		theme_fxmmmenufore  = FXRGB(  0,  0,  0);
  		theme_fxmmmenuback  = FXRGB(130,130,130);
  		theme_fxmmselfore   = FXRGB(  0,  0,  0);
  		theme_fxmmselback   = FXRGB(150,150,150);
  		theme_fxmmtipfore  = FXRGB( 50, 50, 50);
  		theme_fxmmtipback  = FXRGB(210,210,210);
  		theme_fxmmhilite  = FXRGB(200,200,200);
  		theme_fxmmshadow  = FXRGB(110,110,110);
	break;
	case ID_SELECT_THEME_GENTLE:
  		theme="gentle";
  		theme_fxmmfore   = FXRGB(  0,  0,  0);
  		theme_fxmmbase   = FXRGB(160,160,160);
  		theme_fxmmborder = FXRGB( 40, 40, 40);
  		theme_fxmmback   = FXRGB(220,220,220);
  		theme_fxmmmenufore  = FXRGB(  0,  0,  0);
  		theme_fxmmmenuback  = FXRGB(200,200,200);
  		theme_fxmmselfore   = FXRGB(  0,  0,  0);
  		theme_fxmmselback   = FXRGB(180,180,180);
  		theme_fxmmtipfore  = FXRGB( 50, 50, 50);
  		theme_fxmmtipback  = FXRGB(210,210,210);
  		theme_fxmmhilite  = FXRGB(180,180,180);
  		theme_fxmmshadow  = FXRGB(90,90,90);
	break;

	case ID_SELECT_THEME_FOG:
  		theme="fog";
  		theme_fxmmfore   = FXRGB(  0,  0,  0);
  		theme_fxmmbase   = FXRGB(180,180,180);
  		theme_fxmmborder = FXRGB( 50, 50, 50);
  		theme_fxmmback   = FXRGB(230,230,230);
  		theme_fxmmmenufore  = FXRGB(  0,  0,  0);
  		theme_fxmmmenuback  = FXRGB(200,200,200);
  		theme_fxmmselfore   = FXRGB(  0,  0,  0);
  		theme_fxmmselback   = FXRGB(180,180,180);
  		theme_fxmmtipfore  = FXRGB( 50, 50, 50);
  		theme_fxmmtipback  = FXRGB(210,210,210);
  		theme_fxmmhilite  = FXRGB(200,200,200);
  		theme_fxmmshadow  = FXRGB(110,110,110);
	break;
	case ID_SELECT_THEME_MOON:
  		theme="moon";
  		theme_fxmmfore   = FXRGB(200,200,200);
  		theme_fxmmbase   = FXRGB( 60, 60, 60);
  		theme_fxmmborder = FXRGB(  0,  0,  0);
  		theme_fxmmback   = FXRGB( 20, 20, 20);
  		theme_fxmmmenufore  = FXRGB(150,150,150);
  		theme_fxmmmenuback  = FXRGB( 10, 10, 10);
  		theme_fxmmselfore   = FXRGB(200,200,200);
  		theme_fxmmselback   = FXRGB(100,100,100);
  		theme_fxmmtipfore  = FXRGB(200,200,200);
  		theme_fxmmtipback  = FXRGB( 20, 20, 20);
  		theme_fxmmhilite  = FXRGB(100,100,100);
  		theme_fxmmshadow  = FXRGB( 10, 10, 10);
	break;
	}
    return 1;
}


  // Save Colors in the registry file
void MimeTypeDialog::saveThemeColors(){
  getApp()->reg().writeColorEntry("SETTINGS","fxmmbasecolor",theme_fxmmbase);
  getApp()->reg().writeColorEntry("SETTINGS","fxmmforecolor",theme_fxmmfore);
  getApp()->reg().writeColorEntry("SETTINGS","fxmmbackcolor",theme_fxmmback);
  getApp()->reg().writeColorEntry("SETTINGS","fxmmbordercolor",theme_fxmmborder);
  getApp()->reg().writeColorEntry("SETTINGS","fxmmmenuforecolor",theme_fxmmmenufore);
  getApp()->reg().writeColorEntry("SETTINGS","fxmmmenubackcolor",theme_fxmmmenuback);
  getApp()->reg().writeColorEntry("SETTINGS","fxmmselforecolor",theme_fxmmselfore);
  getApp()->reg().writeColorEntry("SETTINGS","fxmmselbackcolor",theme_fxmmselback);
  getApp()->reg().writeColorEntry("SETTINGS","fxmmtipforecolor",theme_fxmmtipfore);
  getApp()->reg().writeColorEntry("SETTINGS","fxmmtipbackcolor",theme_fxmmtipback);
  getApp()->reg().writeColorEntry("SETTINGS","fxmmhilitecolor",theme_fxmmhilite);
  getApp()->reg().writeColorEntry("SETTINGS","fxmmshadowcolor",theme_fxmmshadow);
}


// Set search path for icons
long MimeTypeDialog::onCmdSelectIconPath(FXObject*,FXSelector,void*){
  FXString path=(iconpath.empty()) ? "/usr/local/share/pixmaps" : iconpath.text();
  FXDirDialog opendialog(this,"Search path for icons?");
  opendialog.setDirectory(path);
  if(opendialog.execute()){
    iconpath=opendialog.getDirectory();
    }
  return 1;
  }


// Set terminal
long MimeTypeDialog::onCmdSelectTerminal(FXObject*,FXSelector,void*){
  FXString term=(terminal.empty()) ? "/usr/bin/xterm" : terminal.text();
  FXFileDialog opendialog(this,"Terminal?");
  if(opendialog.execute()){
    terminal=opendialog.getFilename();
    }
  return 1;
  }


// Set command to open the file with
long MimeTypeDialog::onCmdSelectCommand(FXObject*,FXSelector,void*){
  FXString path=(fileassociation.command.empty()) ? "/usr/bin/" : fileassociation.command.text();
  FXString search=FXFileDialog::getOpenFilename(this,"Command to open file with?",path,"*");
  if(!search.empty()){
    fileassociation.command=search;
    }
  return 1;
  }


long MimeTypeDialog::onCmdSelectMimeType(FXObject*,FXSelector,void*ptr){
  FXString content=(FXchar*)ptr;
  if(!content.empty() && (mimetypelist->findItem(content)==-1)){
    mimetypelist->appendItem(content);
    mimetypelist->sortItems();
    }
  fileassociation.mimetype=content;
  return 1;
  }


long MimeTypeDialog::onCmdFileAssociation(FXObject*,FXSelector,void*){
  saveFileAssociation();

  FXIcon *newicon;
  FXString iconname;
  FXString association;

  fileassociation.extension=associationlist->getItemText(associationlist->getCurrentItem());
  if(fileassociation.extension.empty()) return 1;

  association=getApp()->reg().readStringEntry("FILETYPES",fileassociation.extension.text());

  // here is the same association order of ControlPanel to be able to import associations 
  fileassociation.command = association.section(";",0);  // the command to open the file with
  fileassociation.type = association.section(";",1);     // the name of the type
  fileassociation.bigname = association.section(";",2);  // the big icon 
  fileassociation.smallname = association.section(";",3); // the small icon 
  fileassociation.mimetype = association.section(";",4); // the content-type [object/format]

  if(!fileassociation.mimetype.empty()){
    FXint no=mimetypelist->findItem(fileassociation.mimetype);
    mimetypelist->setCurrentItem(no);
    }
  else {
    FXint no=mimetypelist->findItem(" ");
    mimetypelist->setCurrentItem(no);
    }

  if(button_bigname->getIcon()){
    newicon=button_bigname->getIcon();
    button_bigname->setIcon(NULL);
    delete newicon;
    }
  newicon=createIconApp(getApp(),fileassociation.bigname,iconpath);
  button_bigname->setIcon(newicon);

  if(button_smallname->getIcon()){
    newicon=button_smallname->getIcon();
    button_smallname->setIcon(NULL);
    delete newicon;
    }
  newicon=createIconApp(getApp(),fileassociation.smallname,iconpath);
  button_smallname->setIcon(newicon);

  return 1;
  }


long MimeTypeDialog::onCmdCreateFileAssociation(FXObject*,FXSelector,void*){
  FXString result;
  if(FXInputDialog::getString(result,this,"New File Association","Please enter fileassociation extension:",NULL)){
    if(result.empty()) return 1;
    if((associationlist->findItem(result))>=0){
      FXMessageBox::question(this,MBOX_OK,"Duplicate Association","The extension given %s already exists.",result.text());
      return 1;
      }

    // Save Old
    saveFileAssociation();

    fileassociation.extension = result;
    fileassociation.type = "";
    fileassociation.command = "";
    fileassociation.mimetype = "";
    fileassociation.bigname = "";
    fileassociation.smallname = "";

    // Save New
    saveFileAssociation();

    FXint no = associationlist->appendItem(fileassociation.extension);
    associationlist->setCurrentItem(no,TRUE);
    associationlist->sortItems();
    associationlist->makeItemVisible(associationlist->getCurrentItem());
    }
  return 1;
  }


long MimeTypeDialog::onCmdDeleteFileAssociation(FXObject*,FXSelector,void*){
  if(fileassociation.extension.empty()) return 1;
  if(FXMessageBox::question(this,MBOX_OK_CANCEL,"Delete Fileassociation?","Are you sure you want to delete\nthe fileassociation for %s",fileassociation.extension.text())==MBOX_CLICKED_OK){
    getApp()->reg().deleteEntry("FILETYPES",fileassociation.extension.text());
    fileassociation.extension="";
    associationlist->removeItem(associationlist->getCurrentItem(),TRUE);
    }
  return 1;
  }


long MimeTypeDialog::onCmdRenameFileAssociation(FXObject*,FXSelector,void*){
  FXint no;
  FXString result=fileassociation.extension;
  if(FXInputDialog::getString(result,this,"Rename File Association","Rename File Association",NULL)){
    if(result==fileassociation.extension) return 1;
    if((no=associationlist->findItem(result))>=0){
      if(FXMessageBox::question(this,MBOX_YES_NO,"Replace Association?","Would you like to replace the existing association?")!=MBOX_CLICKED_YES){
        return 1;
        }
      }

    // Find Extension to rename
    getApp()->reg().deleteEntry("FILETYPES",fileassociation.extension.text());
    fileassociation.extension=result;
    // Save renamed Extension 
    saveFileAssociation();
    if(no>=0){
      FXint olditem = associationlist->getCurrentItem();
      associationlist->setCurrentItem(no,TRUE);
      // Remove old Extension
      associationlist->removeItem(olditem);
      }
    else {
      associationlist->setItemText(associationlist->getCurrentItem(),fileassociation.extension);
      }
    }
  return 1;
  }


long MimeTypeDialog::onCmdSelectIcon(FXObject*,FXSelector sel,void*){
  FXString name;
    switch(FXSELID(sel)){
	case ID_SELECT_ICONBIG_NAME: name=fileassociation.bigname; break;
	case ID_SELECT_ICONSMALL_NAME: name=fileassociation.smallname; break;
    }

    if(!FXStat::exists(name)){ name=FXPath::search(iconpath,name); }

    if(name.empty()){
      if(!iconpath.empty()){ name=iconpath.section(PATHLISTSEP,0)+PATHSEPSTRING; }
      else{ name=FXSystem::getCurrentDirectory(); }
    }

  FXFileDialog opendialog(this,"Select the Icon");
  opendialog.setSelectMode(SELECTFILE_EXISTING);
  opendialog.setFilename(name);
  opendialog.showImages(TRUE);
  if(opendialog.execute()){

	FXString iconroot=opendialog.getFilename();
	//FXString iconname=FXPath::name(iconroot);

	switch(FXSELID(sel)){
		case ID_SELECT_ICONBIG_NAME:
			updateIconApp(getApp(),button_bigname,iconroot);
			//fileassociation.bigname=iconame;
			fileassociation.bigname=iconroot;
		break;
		case ID_SELECT_ICONSMALL_NAME:
			updateIconApp(getApp(),button_smallname,iconroot);
			//fileassociation.smallname=iconname;
			fileassociation.smallname=iconroot;
		break;
	}
    }
  return 1;
  }


long MimeTypeDialog::onCmdSelectFont(FXObject*,FXSelector,void*){
  FXFontDialog fontdlg(this,"Change Body Font",DECOR_BORDER|DECOR_TITLE);
  FXFontDesc fontdesc=fontbutton->getFont()->getFontDesc();
  fontdlg.setFontDesc(fontdesc);
  if(fontdlg.execute()){
    FXFont *oldfont=bodyfont;
    fontdesc=fontdlg.getFontDesc();
    bodyfont=new FXFont(getApp(),fontdesc);
    bodyfont->create();
    fontbutton->setFont(bodyfont);
    delete oldfont;
    }
  return 1;
  }


// Close the dialog
long MimeTypeDialog::onCmdCloseDialog(FXObject*,FXSelector,void*){

  FXint result=FXMessageBox::question(this,MBOX_SAVE_CANCEL_DONTSAVE,"Save Changes?","Do you want to save changes to the FXMovieManager Registry?\n\nThen you must restart the application for the changes to take effect.");
  if(result==MBOX_CLICKED_CANCEL){ 
	return 1;
  }
  if(result==MBOX_CLICKED_SAVE){

	saveFileAssociation();
	saveThemeColors();
	// Save Settings
  	getApp()->reg().writeIntEntry("SETTINGS","fxmmwheellines",fxmmwheellines);
	getApp()->reg().writeStringEntry("SETTINGS","mimeiconpath",iconpath.text());
  	getApp()->reg().writeStringEntry("SETTINGS","theme",theme.text());
  	getApp()->reg().writeStringEntry("SETTINGS","language",language.text());
  	getApp()->reg().writeStringEntry("SETTINGS","terminal",terminal.text());
        fontspec=fontbutton->getFont()->getFont();
	getApp()->reg().writeStringEntry("SETTINGS","bodyfont",fontspec.text());

#ifdef HAVE_LIBFFMPEGTHUMBNAILER
	// Thumbnailer
  	getApp()->reg().writeIntEntry("THUMBNAILER","thumbnailSize",thumbnailSize);
  	getApp()->reg().writeIntEntry("THUMBNAILER","imageQuality",imageQuality);
  	getApp()->reg().writeBoolEntry("THUMBNAILER","filmStripOverlay",filmStripOverlay);
  	getApp()->reg().writeBoolEntry("THUMBNAILER","workaroundIssues",workaroundIssues);
  	getApp()->reg().writeBoolEntry("THUMBNAILER","maintainAspectRatio",maintainAspectRatio);
  	getApp()->reg().writeBoolEntry("THUMBNAILER","smartFrameSelection",smartFrameSelection);
#endif
  	getApp()->reg().writeBoolEntry("THUMBNAILER","autoThumbnailShow",autoThumbnailShow);
  	getApp()->reg().writeBoolEntry("THUMBNAILER","autoThumbnailCreate",autoThumbnailCreate);
  }
  FXWindow::hide();
  return 1;
  }

// EOF
